#include <stdio.h>
#include <string.h>

#define VERSION                 "0.2"
#define SEQ_BANK_START          0x7B0860 

unsigned char* RomBuffer;
unsigned char* SeqBuffer;
unsigned char* FreeSpace;

int main(int argc, char *argv[]) 
{
    
    if (argc < 4)
    {
             
      printf("\nMario 64 Sequence Inserter %s by messiaen"
             "\nUsage: insert_seq <sequence> <rom_file> <sequence_number> or <-all>"
             "\nCheck readme.txt for sequence numbering and reference\n", VERSION);
             
      getch();
           
      return -1;
    
    }
    
    int i = 0;
    int all_sequences = 0;
    int seq_number = 0;
    int seq_pointer = 0;
    int clean = 0;

    if ( strcmp(argv[3], "-all"  ) == 0 )  all_sequences = 1;

    /* Check if sequence number is valid */
    
    if ( all_sequences == 0 && clean == 0)
    {
      seq_number = strtoul (argv[3],NULL,10);
   
      if (seq_number < 1 || seq_number > 0x23 && all_sequences == 0)
      {
        printf("\nInvalid sequence number. Check insert_seq.txt for details.\n");
        getch();
        return -1;
      }
    } 
    
    /* Open sequence file */
    
    FILE *seq = fopen(argv[1], "rb");
    
    if (seq == 0)
    {
      printf("\nError opening sequence file.\n");
      getch();
      return -1;            
    }
    
    fseek (seq , 0 , SEEK_END);
    int SeqSize = ftell (seq);
    rewind(seq);
    
    SeqBuffer = (char*)malloc(sizeof(char)*SeqSize);
    fread(SeqBuffer, 1, SeqSize,  seq);
    
    fclose(seq);
    
    /* Open ROM file */    
    
    FILE *rom = fopen(argv[2], "rb");
    
    if (rom == 0)
    {
      printf("\nError opening ROM file.\n");
      getch();
      return -1;            
    }
    
    fseek (rom , 0 , SEEK_END);
    int RomSize = ftell (rom);
    
    if (!(RomSize > 0x1600000)) 
    {
      printf("\nNot an extended Super Mario 64 (US) 1.0 ROM.\n");
      getch();
      return -1;
    }
                  
    rewind(rom);
    
    int RomBufferSize = 0x800000 - SEQ_BANK_START + 1;
    
    RomBuffer = (char*)malloc(sizeof(char)*RomBufferSize);
    fseek(rom, SEQ_BANK_START, SEEK_SET);
    fread(RomBuffer, 1, RomBufferSize, rom);
    
    fclose(rom);
    
    /* Search free space in ROM (an 0xFF array) */       
    
    FreeSpace = (char*)malloc(sizeof(char*)*SeqSize);
    
    for (i = 0; i < SeqSize; i++)
    {
      FreeSpace[i] = 0xFF;
    }
               
    for( i = 0; i < RomBufferSize; i++ ) 
    {
      if( !memcmp(RomBuffer + i, FreeSpace, SeqSize) ) 
      { 
        seq_pointer = i;
          if (seq_pointer % 0x10 == 0)      /* Check if padding is correct */
          {
            printf( "\nFound free space at 0x%06x", seq_pointer + SEQ_BANK_START);
            printf( "\nSequence pointer = 0x%06x", seq_pointer);
            break;
          }
      }
    }
    
    if (seq_pointer == 0)
    {
      printf("\nNot enough space in ROM for sequence file. Use seq_clean.\n");
      getch();
      return -1;
    }

    /* Write sequence to ROM */   
   
    rom = fopen(argv[2], "r+b");
    fseek(rom, seq_pointer + SEQ_BANK_START, SEEK_SET);
    fwrite(SeqBuffer, 1, SeqSize, rom);
     
    /* Adjust pointers */       
              
    if (all_sequences == 0)
    {   
      fseek(rom, SEQ_BANK_START + (seq_number * 8) + 4, SEEK_SET);

      fputc(seq_pointer >> 24, rom);
      fputc(seq_pointer >> 16, rom);
      fputc(seq_pointer >> 8, rom);
      fputc(seq_pointer, rom);
    
      fputc(SeqSize >> 24, rom);
      fputc(SeqSize >> 16, rom);
      fputc(SeqSize >> 8, rom);
      fputc(SeqSize, rom);
     
      printf("\nSequence #%d sucessfully inserted at 0x%06x!\n", seq_number, SEQ_BANK_START + seq_pointer);
    
    }
    else
    {
      /* Replace all sequence */
      
      for (i = 1; i < 0x23; i++)
      {
        fseek(rom, SEQ_BANK_START + (i * 8) + 4, SEEK_SET);

        fputc(seq_pointer >> 24, rom);
        fputc(seq_pointer >> 16, rom);
        fputc(seq_pointer >> 8, rom);
        fputc(seq_pointer, rom);
      
        fputc(SeqSize >> 24, rom);
        fputc(SeqSize >> 16, rom);
        fputc(SeqSize >> 8, rom);
        fputc(SeqSize, rom);
      }
        
      printf("\nSequence sucessfully inserted at 0x%06x!\n", SEQ_BANK_START + seq_pointer);
    }
    
    printf("\nPress any key to return ...");
    getch();
             
    fclose(rom);
    fclose(seq);
    free(SeqBuffer);
    free(RomBuffer);
    free(FreeSpace);
    
  	return 0;
  	
  	/* ToDo: this program has growed up so it REALLY needs to be redesigned */ 
  	
}
