#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "z64parser.h"

#define VERSION  "v0.1"

/* Function Prototypes */

void AddMusicTrack(int MusicTrackChan, int MusicTrackOffset1, int MusicTrackOffset2);
void AddJump(int source_offset, int dest_offset1, int dest_offset2);
void AddTrack(int TrackChan, int TrackOffset1, int TrackOffset2);
void ParseHeader(FILE* input);
void ParseMusicTrack(int MusicTrackOffset, FILE* input);
void ParseTrack(int TrackNum, FILE* input);
void PrintMusicEvent(int note, int transposition, int type, int param1, int param2, int param3);
void PrintTimestamp (int timestamp);
void PrintChannels(int param);

/* Global variables and structs*/

int num_tracks;
int music_tracks;
int num_jumps;

struct Track
{
int num;
int chan;
int layer;
int offset;
        
} TrackInfo[100], MusicTrackInfo[2000]; /* semi-random numbers (ToDo: create linked-lists for this)*/

struct Jump             /* for music command 0xFC */
{
int num;
int source_offset;
int dest_offset;
int redundant;
} JumpList[1000];

/* Functions */

void PrintMusicEvent(int note, int transposition, int type, int param1, int param2, int param3)
{
      
int input_byte, octave, pitch, timestamp, velocity, duration;
char pitch_str[4];
char timestamp_str[40];

input_byte = note;
note = note + transposition;
      
switch (type) {
       
       case 0x00:             /* 0x00 to 0x3F -> format = note, timestamp, velocity, duration (related to "NoteOff") */
       note -= 3;
       octave = (note / 12) + 1;
       pitch = note % 12;
       timestamp = param1;
       velocity = param2;
       duration = param3;
       break;
       
       case 0x01:
       note -= 3;
       octave = ((note - 0x40) / 12) + 1;
       pitch = (note - 0x40) % 12;
       timestamp = param1;            
       duration = 0;                    
       break;
       
       case 0x02:
       note -= 3;
       octave = ((note - 0x80) / 12) + 1;
       pitch = (note - 0x80) % 12;
       timestamp = 0;
       velocity = param1;
       duration = param2;             /* I'm not entirely sure about this */
       break;
       
       }
       
switch (pitch) {
       
       case 0:
       strcpy(pitch_str, "C ");       
       break;
       
       case 1:
       strcpy(pitch_str, "C#");
       break;

       case 2:
       strcpy(pitch_str, "D ");      
       break;

       case 3:
       strcpy(pitch_str, "D#");     
       break;

       case 4:
       strcpy(pitch_str, "E ");            
       break;

       case 5:
       strcpy(pitch_str, "F ");  
       break;

       case 6:
       strcpy(pitch_str, "F#"); 
       break;

       case 7:
       strcpy(pitch_str, "G ");
       break;

       case 8:
       strcpy(pitch_str, "G#"); 
       break;

       case 9:
       strcpy(pitch_str, "A "); 
       break;

       case 10:
       strcpy(pitch_str, "A#");           
       break;

       case 11:
       strcpy(pitch_str, "B ");                   
       break;
              
       }

switch (timestamp) {
       
       case SIXTEEN_MEASURES:
       strcpy(timestamp_str, "Sixteen 4/4 measures");            
       break;

       case TWELVE_MEASURES:
       strcpy(timestamp_str, "Twelve 4/4 measures");            
       break;

       case EIGHT_MEASURES:
       strcpy(timestamp_str, "Eight 4/4 measures");            
       break;

       case FOUR_MEASURES:
       strcpy(timestamp_str, "Four 4/4 measures");            
       break;

       case TWO_MEASURES:
       strcpy(timestamp_str, "Two 4/4 measures");            
       break;
       
       case WHOLE_NOTE:
       strcpy(timestamp_str, "Whole-note");            
       break;
       
       case DOTTED_HALF_NOTE:
       strcpy(timestamp_str, "Dotted Half-note");
       break;
       
       case HALF_NOTE:
       strcpy(timestamp_str, "Half-note");
       break;
       
       case DOTTED_QUARTER:
       strcpy(timestamp_str, "Dotted Quarter");            
       break;
       
       case QUARTER_NOTE:
       strcpy(timestamp_str, "Quarter note");
       break;
       
       case QUARTER_TRIPLET:
       strcpy(timestamp_str, "Quarter-note Triplet");   
       break;         
      
       case DOTTED_EIGHT:
       strcpy(timestamp_str, "Dotted Eight-note");            
       break;
       
       case EIGHT_NOTE:
       strcpy(timestamp_str, "Eight-note");            
       break;
       
       case EIGHT_TRIPLET:
       strcpy(timestamp_str, "Eight-note Triplet");            
       break;
       
       case SIXTEENTH:
       strcpy(timestamp_str, "16th note");            
       break;
       
       default:
       strcpy(timestamp_str, "Other");      
       break;       
       
       }       
       
  
switch (type) {
       
       case 0x00:
       printf ("Cmd 0x%02x | Note %s%d | Timestamp: %s (0x%x) | Velocity: 0x%02x | Duration: 0x%02x\n", input_byte, pitch_str, octave, timestamp_str, timestamp, velocity, duration);
       break;
       
       case 0x01:
       printf ("Cmd 0x%02x | Note %s%d | Timestamp: %s (0x%x) | Velocity: 0x%02x\n", input_byte, pitch_str, octave, timestamp_str, param1, param2);            
       break;
       
       case 0x02:
       printf ("Cmd 0x%02x | Note %s%d | Timestamp: (same as previous one) | Velocity: 0x%02x | Duration: 0x%02x\n", input_byte, pitch_str, octave, velocity, duration);            
       break;
       
       }       


     
}

void AddJump(int source_offset, int dest_offset1, int dest_offset2)
{
   int i, j;
   
   num_jumps++;
   JumpList[num_jumps].num = num_jumps;    
   JumpList[num_jumps].source_offset = source_offset;     /* current position */
   JumpList[num_jumps].dest_offset = (dest_offset1 << 8) + dest_offset2;
   JumpList[num_jumps].redundant = 0;
   
   
}

void AddTrack(int TrackChan, int TrackOffset1, int TrackOffset2)
{

   num_tracks++;           /* current track, also used for track counting */            
   TrackInfo[num_tracks].num = num_tracks;
   TrackInfo[num_tracks].chan = TrackChan;   
   TrackInfo[num_tracks].offset = (TrackOffset1 << 8) + TrackOffset2;
  
}

void AddMusicTrack(int MusicTrackLayer, int MusicTrackOffset1, int MusicTrackOffset2)
{

   music_tracks++;
   MusicTrackInfo[music_tracks].num = music_tracks;
   MusicTrackInfo[music_tracks].layer = MusicTrackLayer;
   MusicTrackInfo[music_tracks].offset = (MusicTrackOffset1 << 8) + MusicTrackOffset2;
  
}

void PrintTimestamp (int timestamp)
{
     
     switch (timestamp) {
       
       case SIXTEEN_MEASURES:
       printf("Sixteen 4/4 measures");            
       break;

       case TWELVE_MEASURES:
       printf("Twelve 4/4 measures");            
       break;

       case EIGHT_MEASURES:
       printf("Eight 4/4 measures");            
       break;

       case FOUR_MEASURES:
       printf("Four 4/4 measures");            
       break;

       case TWO_MEASURES:
       printf("Two 4/4 measures");            
       break;
       
       case WHOLE_NOTE:
       printf("Whole-note");            
       break;
       
       case DOTTED_HALF_NOTE:
       printf("Dotted Half-note");
       break;
       
       case HALF_NOTE:
       printf("Half-note");
       break;
       
       case DOTTED_QUARTER:
       printf("Dotted Quarter");            
       break;
       
       case QUARTER_NOTE:
       printf("Quarter note");
       break;
       
       case QUARTER_TRIPLET:
       printf("Quarter-note Triplet");   
       break;         
      
       case DOTTED_EIGHT:
       printf("Dotted Eight-note");            
       break;
              
       case EIGHT_NOTE:
       printf("Eight-note");            
       break;
       
       case EIGHT_TRIPLET:
       printf("Triplet");            
       break;
       
       case SIXTEENTH:
       printf("16th note");            
       break;
       
       default:
       printf("Other");      
       break;       
       
       }
}

void PrintChannels(int param)

{
     
    int channels[16];     
  	unsigned int i;
    int j, x;
  	
  	i = 1<<(2 * 8 - 1);  /* 16 bits */
  
    x = 16;
  	for (j = 0; j < 16; j++)
 	{
        x--;
  		if (param & i)
  			{ 
            channels[x] = 1;
//          printf("1");
            }
  		else
            {
//			printf("0");
            channels[x] = 0;  			
            } 
  			
        i >>= 1;
        	
     }
     
     printf("Channels: ");
     

  	for (j = 0; j < 16; j++)     
	{
    if (channels[j] == 1) printf("%d ", j); 
    }     
  
}    

 

void ParseMusicTrack(int MusicTrackOffset, FILE* input)
{
int byte = 0;
int param1 = 0;
int param2 = 0;
int param3 = 0;
int x = 0; 
int pos = 0;
int first_nibble = 0;
int second_nibble = 0;
pos = (MusicTrackOffset) - 1;
int transposition = 0;

fseek(input,MusicTrackOffset, SEEK_SET);

while (byte != 0xFF && byte != EOF)
      { 
      byte = fgetc(input);
      pos++;
     
      first_nibble  = byte >> 4;
      second_nibble = byte & 0x0F;

      
      if (byte >= 0 && byte <= 0x3F) { 
                       
         printf ("0x%x: ", pos);
         
         param1 = fgetc(input); /* timestamp, variable lenght */
         
         int param1_secondbyte, param1_first_nibble, param1_second_nibble;
         
         param1_first_nibble  = (param1 >> 4);
         param1_second_nibble = (param1 & 0x0F);
         
         if (param1_first_nibble == 8) {
                          
             param1_secondbyte = fgetc(input);
             param1 = ( (param1_second_nibble << 8) + param1_secondbyte);
             pos++;
                          
         }

         param2 = fgetc(input);
         param3 = fgetc(input); 
         
         PrintMusicEvent(byte, transposition, 0, param1, param2, param3);  /* type = 0 */
         
         pos += 3;
         
         }
         
      else if (byte >= 0x40 && byte <= 0x7F) 
                       
               {
               printf ("0x%x: ", pos);
               
               param1 = fgetc(input); /* variable-lenght timestamp */
               
               int param1_secondbyte, param1_first_nibble, param1_second_nibble;
         
               param1_first_nibble  = (param1 >> 4);
               param1_second_nibble = (param1 & 0x0F);
         
               if (param1_first_nibble == 8) {
                           
               param1_secondbyte = fgetc(input);
               param1 = ( (param1_second_nibble << 8) + param1_secondbyte);
               pos++;
                          
               }
               
               param2 = fgetc(input); /* not sure */
               
               PrintMusicEvent(byte, transposition, 1, param1, param2, 0); /* type = 1 */
               
               pos += 2;
         
       }
         
      else if (byte >= 0x80 && byte <= 0xBF) { 

              printf ("0x%x: ", pos);    
              
              param1 = fgetc(input);
              param2 = fgetc(input);
              
              PrintMusicEvent(byte, transposition, 2, param1, param2, 0); /* type = 2 */
               
              pos += 2;

      }            
      
      /* This is never going to happen */
      
      else if (first_nibble == 8) {
           
           if (second_nibble == 0) { 
               param1 = fgetc(input);
               printf("0x%x: 80 %02x --> Timestamp (rest)\n", pos, param1);                  
               pos++;
           }

           else {
                  param1 = fgetc(input);
                  printf("0x%x: 8%x %02x --> Timestamp (rest): ", pos, second_nibble, param1 ); 
                  PrintTimestamp( second_nibble << 8 + param1 );
                  printf(" (0x%x\n", ( (second_nibble << 8) + param1) ); 
                  
                  pos++;                  
                
            }
     
     
      }               
                      
     else {

           /* process other events */
                 
           switch (byte)  {

                  case 0xC0:
                       
                  param1 = fgetc(input);
                  
                  int param1_secondbyte, param1_first_nibble, param1_second_nibble;
         
                  param1_first_nibble  = (param1 >> 4);
                  param1_second_nibble = (param1 & 0x0F);
         
                  if (param1_first_nibble == 8) {
                          
                  param1_secondbyte = fgetc(input);
                  printf("0x%x: C0 %02x %02x --> Rest: ", pos, param1, param1_secondbyte);
                  param1 = ( (param1_second_nibble << 8) + param1_secondbyte );
                  PrintTimestamp(param1);
                  printf(" (0x%x)\n", param1);
                  pos += 2;
                  }
                  else { 
                       printf("0x%x: C0 %02x --> Rest: ", pos, param1 );
                       PrintTimestamp(param1);
                       printf(" (0x%02x)\n", param1);
                       pos ++; }
                  
                  break;
                  
                  case 0xC2: /* music track transposition */
                  param1 = fgetc(input);
                  printf("0x%x: C2 %02x --> Track transposition = %d semitones \n", pos, param1, param1 );
                  transposition = param1;
                  pos++;
                  break;
                        
                  case 0xE0:
                  printf("0x%x: E0 (no param)\n", pos);
                  break;
                  
                  case 0xE4:
                  printf("0x%x: E0 (no param)\n", pos);
                  break;
                  
                  case 0xE7:
                  printf("0x%x: E7 (no param)\n", pos);                      
                  break;
                  
                  case 0xE8:
                  printf("0x%x: E8 (no param)\n", pos);                      
                  break;
                  
                  case 0xFC: /* Jump (used in loops) */
                  param1 = fgetc(input);
                  param2 = fgetc(input);
                  printf("0x%x: FC %02x %02x --> Jump to offset 0x%x\n", pos, param1, param2, ( (param1 << 8) + param2) );
                  
                  AddJump(pos, param1, param2);
                  
                  pos += 2;
                  break;
                       
                  case 0xFF:
                  printf("--- END OF MUSIC TRACK AT 0x%x --- \n\n", pos );
                  break;

                  default:
                  printf("0x%x: Unknown command = %02x\n", pos, byte);
                  break;          
             
                  } 
             
          }
       
     }
      
}
     
void ParseTrack(int TrackNum, FILE* input)
{
int byte = 0;
int param1 = 0;
int param2 = 0;
int param3 = 0;
int x = 0; 
int pos = 0;
int first_nibble = 0;
int second_nibble = 0;
pos = (TrackInfo[TrackNum].offset) - 1;

fseek(input,TrackInfo[TrackNum].offset, SEEK_SET);

while (byte != 0xFF && byte != EOF)
      { 
      byte = fgetc(input);
      pos++;
      
      first_nibble  = byte >> 4;
      second_nibble = byte & 0x0F;
      
      if (first_nibble == 9) {
                       
             param1 = fgetc(input);
             param2 = fgetc(input);
             printf("0x%x: 9%x %02x %02x --> Load Music Track (Layer %d) from offset 0x%x\n", pos, second_nibble, param1, param2, second_nibble, ( (param1 << 8) + param2) );
             AddMusicTrack(second_nibble, param1, param2);
             pos += 2;
                       
         }
         
      if (first_nibble == 8) {
                       
             param1 = fgetc(input);
             param2 = fgetc(input);
             printf("0x%x: 8%x %02x %02x --> Load Music Track (Layer %d) from offset 0x%x\n", pos, second_nibble, param1, param2, second_nibble, ( (param1 << 8) + param2) );
             AddMusicTrack(second_nibble, param1, param2);
             pos += 2;
                       
         }         
                       
      else {
      
      switch (byte)  {

             case 0xC1: /* SetProgram related */
             param1 = fgetc(input);
             printf("0x%x: C1 %02x --> Set Program = %d ", pos, param1, param1); 
             
             if (param1 == 0x7f)  printf("(drums)");
             
             printf("\n");
             pos++;
             break;
             
             case 0xC4: /* Marks track begin? */
             printf("0x%x: C4 --> Begin track\n", pos);
             break;
             
             case 0xDC: /* Drum-related? */
             param1 = fgetc(input);             
             printf("0x%x: DC %02x --> Drum-related?\n", pos, param1, param1);
             pos++;
             break;

             case 0xD3:
             param1 = fgetc(input);
             printf("0x%x: D3 %02x --> Pitch bend = 0x%02x \n", pos, param1, param1);
             pos++;
             break;
             
             case 0xD4:
             param1 = fgetc(input);
             printf("0x%x: D4 %02x --> ?\n", pos, param1);
             pos++;
             break;             
             
             case 0xD9:
             param1 = fgetc(input);
             printf("0x%x: D8 %02x --> ?\n", pos, param1);
             pos++;
             break;             
             

             case 0xDD: /* Pan */
             param1 = fgetc(input);
             printf("0x%x: DD %02x --> Set Track Pan = %d\n", pos, param1, param1);
             pos++;
             break;
             
             case 0xE9: /* Not-sure */
             param1 = fgetc(input);
             printf("0x%x: E9 %02x --> Initializes something (usual param = 02)\n", pos, param1);
             pos++;
             break;
             
             case 0xFC:  /* jump */
             param1 = fgetc(input);
             param2 = fgetc(input);
             printf("0x%x: FC %02x %02x --> Jump to offset 0x%x\n", pos, param1, param2, ( (param1 << 8) + param2) );
             pos += 2;                  
             break;
                  

             case 0xDF: /* Track volume */
             param1 = fgetc(input);
             printf("0x%x: DF %02x --> Track Volume = %d\n", pos, param1, param1);
             pos++;
             break;
             
             case 0xFD: 
             param1 = fgetc(input);             
             printf("0x%x: FD %02x ", pos, param1);
             pos++;
             
             int param1_secondbyte, param1_first_nibble, param1_second_nibble;
         
             param1_first_nibble  = (param1 >> 4);
             param1_second_nibble = (param1 & 0x0F);
         
             if (param1_first_nibble >= 8) {
                          
             param1_secondbyte = fgetc(input);
             param1 = ( (param1_second_nibble << 8) + param1_secondbyte);
             pos++;
             
             printf("%02x ", param1_secondbyte);
                          
             }
             
             printf("--> Timestamp: ");             
             PrintTimestamp(param1);
             printf(" (0x%x)\n", param1);
             break;                          
                                
             case 0xFF:
             printf("--- END OF TRACK %d AT 0x%x --- \n\n", TrackInfo[TrackNum].num, pos);                  
             break;
             
             default:
             printf("0x%x: Unknown command = %02x\n", pos, byte);
             break;          
       
             }   
        
          }
         
      }      
     
}

void ParseHeader(FILE* input)
{
rewind(input);

int byte = 0;             /* read from file */
int param1;
int param2;
int param3;
int x; 
int pos = -1;         /* current position */
int first_nibble = 0;
int second_nibble = 0;


while (byte != 0xFF && byte != EOF)
      { 
      byte = fgetc(input);
      pos++;
      
      first_nibble  = byte >> 4;
      second_nibble = byte & 0x0F;
      
      if (first_nibble == 9) {
                       
             param1 = fgetc(input);
             param2 = fgetc(input);
             printf("0x%x: 9%x %02x %02x --> Load track from offset 0x%x (Channel %d)\n", pos, second_nibble, param1, param2, ( (param1 << 8) + param2 ), second_nibble );
             AddTrack(second_nibble, param1, param2);
             pos = pos + 2;
                       
         }
                       
      else {      
      
      switch (byte)  {

             case 0xD3:
             param1 = fgetc(input);
             printf("0x%x: D3 %02x --> Begin Header (known types: 80, 60, 20, E0)\n", pos, param1);
             pos++;
             break;
             
             case 0xD5:
             param1 = fgetc(input);
             printf("0x%x: D5 %02x --> Intializes something (usual param = 32)\n", pos, param1);
             pos++;
             break;
             
             case 0xD6:  /* disable channels */
             param1 = fgetc(input);
             param2 = fgetc(input);
             printf("0x%x: D6 %02x %02x --> Disable ", pos, param1, param2);
             PrintChannels( (param1 << 8) + param2);
             printf("\n");
             pos += 2;
             break;
             
             case 0xD7: /* enable channels */
             param1 = fgetc(input);
             param2 = fgetc(input);
             printf("0x%x: D7 %02x %02x --> Enable ", pos, param1, param2);
             PrintChannels( (param1 << 8) + param2);
             printf("\n");
             pos += 2;
             break;             
             
             case 0xDD: /* tempo setting */
             param1 = fgetc(input);
             printf("0x%x: DD %02x --> Tempo = %d (BMP)\n", pos, param1, param1);
             pos++;
             break;
             
             case 0xDB: /* master volume */
             param1 = fgetc(input);
             printf("0x%x: DB %02x --> Master Volume = 0x%02x \n", pos, param1, param1 );             
             pos++;
             break;
             
             case 0xF5:  /* not sure, probably 2 params */ 
             printf("0x%x: Unimplemented 0xF5. Next two bytes should be unknown\n", pos);
             break;

             case 0xFB: /* header loop */
             param1 = fgetc(input);
             param2 = fgetc(input);
             int param = (param1 << 8) + param2;
             printf("0x%x: FB %02x %02x --> Loop from offset 0x%x \n", pos, param1, param2, param);
             pos += 2;
             break;
             
             case 0xFD: /* time-stamp */
             param1 = fgetc(input);             
             printf("0x%x: FD %02x ", pos, param1);
             pos++;
             
             int param1_secondbyte, param1_first_nibble, param1_second_nibble;
         
             param1_first_nibble  = (param1 >> 4);
             param1_second_nibble = (param1 & 0x0F);
         
             if (param1_first_nibble >= 8) {
                          
             param1_secondbyte = fgetc(input);
             param1 = ( (param1_second_nibble << 8) + param1_secondbyte);
             pos++;
             
             printf("%02x ", param1_secondbyte);
                          
             }
             
             printf("--> Timestamp: ");             
             PrintTimestamp(param1);
             printf(" (0x%x)\n", param1);

             break;
             
             case 0xFF:
             printf("--- END OF HEADER AT 0x%x --- \n\n", pos);                  
             break;
             
             default:
             printf("0x%x: Unknown command = %02x\n", pos, byte);
             break;          
       
             }
              
           }  
      }      
}


/* entry-point */

int main ( int argc, char *argv[] )
{
    
int i,j;

FILE *input_file = fopen( argv[1], "rb" );

if (! (input_file == NULL) )  {
      
    num_tracks = 0; 
    music_tracks = 0;  
    num_jumps = 0;   
    
   /* Parse header and search for tracks */ 
   
    printf("Parsed file generated with Zelda 64 parser %s by messiaen\n", VERSION);    
    printf("http://sites.google.com/site/messiaen64\n\n");    

    printf("------\n");    
    printf("HEADER\n");
    printf("------\n\n");
        
    ParseHeader(input_file);
    
    printf("[TRACKS FOUND: %d]\n\n", num_tracks);  
    
   
    /* Parse tracks */ 
    
    for (i=1; i <= num_tracks; i++)  {
        printf("-----------------------------------\n");
        printf("TRACK %d (CHANNEL %d) AT OFFSET 0x%x\n", TrackInfo[i].num, TrackInfo[i].chan, TrackInfo[i].offset);
        printf("-----------------------------------\n\n");        
        ParseTrack(i, input_file);
        }
    
    /* Parse music data */
    
    printf("[MUSIC TRACKS FOUND: %d]\n\n", music_tracks);  
    
    
     for (i=1; i <= music_tracks; i++)  {
          printf("---------------------------------------------\n");         
          printf("MUSIC TRACK %d (LAYER %d) AT OFFSET 0x%x\n", MusicTrackInfo[i].num, MusicTrackInfo[i].layer, MusicTrackInfo[i].offset);
          printf("---------------------------------------------\n\n");          
          ParseMusicTrack(MusicTrackInfo[i].offset, input_file);
          }
          
      /* Check for duplicate jumps before parsing and get rid of them */
                               
      for (i=1; i <= num_jumps; i++) {

          for (j=1 ; j <= num_jumps; j++) {
              
                if ( JumpList[i].dest_offset == JumpList[j].dest_offset && (!(i == j)) && JumpList[i].redundant == 0) JumpList[j].redundant = 1; 
              
              }
          
          }
      
      /* Parse Jumps inside music data*/

     for (i=1; i <= num_jumps; i++)  {
          
          if ( !(JumpList[i].redundant == 1) ) { 
               
              printf("---------------------------------------------\n");         
              printf("JUMP COMMAND #%d AT OFFSET 0x%x. DESTINATION: 0x%x\n", JumpList[i].num, JumpList[i].source_offset, JumpList[i].dest_offset);
              printf("---------------------------------------------\n\n");          

              ParseMusicTrack(JumpList[i].dest_offset, input_file);
              }                   
          }
          
    printf("\nParsed file generated with Zelda 64 parser %s by messiaen\n", VERSION);    
    printf("http://sites.google.com/site/messiaen64\n");    
    
    fclose (input_file);
    }

   else  { 
      printf ("Zelda 64 sequenced music parser %s by messiaen\n", VERSION);
      printf ("Usage: z64parser.exe seq_file > output.txt\n");
//      printf ("Use seq_rip.exe to rip sequences from ROM\n");      
   }
   
}

