#include <stdio.h>

#define PROGRAM_NAME "Zelda 64 Sequence Ripper"
#define VERSION "v0.1b"


/* Function prototypes */

int check_file (FILE* input, int pointers_offset);
int get16 (FILE *input);
int get32 (FILE *input);
int rip_sequences (FILE *input, int seqcount, int sequence_bank_offset, int pointers_offset);

/* Global variables and structs */

int seq_count;

struct sequence {
       int start_offset;
       int lenght;
       int seq_type;   /* 0x0200, 0x0201 or 0x0202 */
       } sequence_list[0x100]; /* this should be updated to a linked-list */

/* Functions */

int check_file (FILE* input, int pointers_offset)
{
    
char ch1, ch2, ch3, ch4;

fseek(input, pointers_offset, SEEK_SET);
ch1 = getc(input);
ch2 = getc(input);
ch3 = getc(input);
ch4 = getc(input);

  if ( ch1 == 0 && (ch2 > 0x30) && ch3 == 0 && ch4 == 0)   {   /* not the best way */
       
     return 1;    /* valid file && seq bank offset */
       
     }
     
  else  {
        return 0; /* wrong file && seq bank offset */
        }
    
}

int get32 (FILE *input)
{
    int byte1, byte2, byte3, byte4;
    int unsigned32;

    byte1 = fgetc(input);
    byte2 = fgetc(input);
    byte3 = fgetc(input);
    byte4 = fgetc(input);  
    
    unsigned32 = (byte1 << 24) + (byte2 << 16) + (byte3 << 8) + byte4;              
    
    return unsigned32;

}

int get16 (FILE *input)
{
    int byte1, byte2;
    int unsigned16;

    byte1 = fgetc(input);
    byte2 = fgetc(input);
    
    unsigned16 = (byte1 << 8) + byte2;
    
    return unsigned16;

}


int rip_sequences (FILE *input, int seqcount, int sequence_bank_offset, int pointers_offset)
{
    
char filename[14];
char ch;
int i, j;
int offset;
  
fseek(input, ((pointers_offset) + 0x10), SEEK_SET);  
  
    for (i = 0; i < seqcount; i++)   {
    
    fseek(input, ((pointers_offset) + 0x10 + (0x10 * i) ), SEEK_SET); 
    
    sequence_list[i].start_offset = get32(input);
   
    sequence_list[i].lenght = get32(input);

    sequence_list[i].seq_type = get16(input);
    
    get16(input);
    get32(input);
    
    }

    for (i = 0; i < seqcount; i++)   {
    
        sprintf(filename, "seq_%02x_%04x.zseq", i, sequence_list[i].seq_type);
    
    FILE *ripped_file = fopen( filename , "w+b"); 
    
        fseek(input, (sequence_bank_offset + sequence_list[i].start_offset) , SEEK_SET);        
   
        for ( j = 0; j < sequence_list[i].lenght; j++)    {
        
            ch = getc(input);
            putc(ch, ripped_file);
            
            }

         fclose(ripped_file);
        
        }
        

}


/* entry-point */

int main ( int argc, char *argv[] )
{
    
if ( argc != 4 ) {
     
   printf(" \n%s %s by messiaen", PROGRAM_NAME, VERSION);
   printf( "\nUsage: z64seqrip.exe [debugrom] [seq_bank_offset] [seq_pointers_offset]" );
   printf( "\nExample: z64seqrip.exe zelootma.z64 0x44DF0 0xBCC6A0\n" );
 
   } 
   
else 
   {

    int sbank_offset = 0;
    sbank_offset = strtoul (argv[2],NULL,0);

    int seq_pointers_offset = 0;    
    seq_pointers_offset = strtoul (argv[3],NULL,0);

    FILE *input_file = fopen( argv[1], "rb" );
   
    if (!(input_file == 0 ))  {
   
      if (!( check_file(input_file, seq_pointers_offset) == 0) )   {                 
                    
         fseek(input_file, (seq_pointers_offset + 1), SEEK_SET);
         seq_count = fgetc(input_file);   
         printf("\n%s %s by messiaen", PROGRAM_NAME, VERSION);         
         printf("\nRipping %d sequences ... ", seq_count);
           
         rip_sequences(input_file, seq_count, sbank_offset, seq_pointers_offset);
         printf("done! \n");
         
         }
       
      else {
        
           printf ("\nNot a valid sequence bank offset.\n");
        
           }
    
      }
      
      else {
           
           printf ("\nError opening input file.\n");
        
           }      
      
    
   }
   
   
}
